#!/bin/sh
#
# Startup script for the EXPRESSCLUSTER webmanager server
#

#
# main()
#

# Source function library.
if [ -f /etc/rc.d/init.d/functions ]
then
	. /etc/rc.d/init.d/functions
elif [ -f /etc/rc.status ]
then
	. /etc/rc.status
fi

. /opt/nec/clusterpro/bin/clpfunctions

PATH=/sbin:/bin:/usr/sbin:/usr/bin:/opt/nec/clusterpro/bin
LD_LIBRARY_PATH=/opt/nec/clusterpro/lib
MALLOC_CHECK_=0
LANG=C

export PATH
export LD_LIBRARY_PATH
export MALLOC_CHECK_
export LANG

# See how we were called.
case "$1" in
  start)
	targetos=`/bin/uname`
	clp_filedel "$1" init_webmgr $targetos
	clp_logwrite "$1" "clusterpro_webmgr start process start." init_webmgr

 	#
	cd /opt/nec/clusterpro/bin

	#
	if [ "$targetos" = "SunOS" ]
	then
		echo "Starting clusterpro webmanager server: \c"
	else
		logging=`clpcfget -g /root/event/container/logging` 
		cfget_ret=$?
		if [ "$cfget_ret" = "0" -a "$logging" = "1" ]
		then
			echo -n "Starting clusterpro webmanager server: " > /dev/null 2>&1
		else
			echo -n "Starting clusterpro webmanager server: "
		fi
	fi
	#
	clp_logwrite "$1" "clppidof start." init_webmgr
	pids=`/opt/nec/clusterpro/bin/clppidof /opt/nec/clusterpro/bin/clpmonp --webmgr 2>&1`
	clp_logwrite "$1" "clppidof end.(pids=${pids})" init_webmgr
	
	if [ "$pids" != "" ]
	then
		clp_logwrite "$1" "webmgr already exist." init_webmgr
		clp_logwrite "$1" "clusterpro_webmgr start process end." init_webmgr
		if [ "$cfget_ret" = "0" -a "$logging" = "1" ]
		then
			clp_success $targetos > /dev/null 2>&1
		else
			clp_success $targetos
		fi
		exit 0
	fi

	# starting clusterpro webmanager server
	clp_logwrite "$1" "webmgr start process start." init_webmgr
	./clpmonp --webmgr > /dev/null 2>&1
	monp_ret=$?
	clp_logwrite "$1" "webmgr start process end.("$monp_ret")" init_webmgr

	if [ "$cfget_ret" = "0" -a "$logging" = "1" ]
	then
		clp_checkerr "$monp_ret" $targetos > /dev/null 2>&1
	else
		clp_checkerr "$monp_ret" $targetos
	fi
	chk_ret=$?
	if [ "$chk_ret" = "1" ]
	then
		clp_logwrite "$1" "clusterpro_webmgr start process end." init_webmgr
		exit 0
	fi

	# PIDFile for systemd
	[ ! -d ../work/service ] && { mkdir -m 0755 ../work/service >/dev/null 2>&1; }
	[ -f ../work/service/clusterpro_webmgr.pid ] && { rm -f ../work/service/clusterpro_webmgr.pid >/dev/null 2>&1; }

	pids=`/opt/nec/clusterpro/bin/clppidof /opt/nec/clusterpro/bin/clpmonp --webmgr 2>&1`

	if [ "$pids" != "" ]
	then
		echo $pids > /opt/nec/clusterpro/work/service/clusterpro_webmgr.pid 2>&1
		clp_logwrite "$1" "create pid file.("$pids")" init_webmgr
	fi

	clp_logwrite "$1" "clusterpro_webmgr start process end." init_webmgr
	;;
  stop)
	targetos=`/bin/uname`

	clp_filedel "$1" init_webmgr $targetos
	clp_logwrite "$1" "clusterpro_webmgr stop process start." init_webmgr

	#
	if [ "$targetos" = "SunOS" ]
	then
		echo "Shutting down clusterpro webmanager server: \c"
	else
		logging=`clpcfget -g /root/event/container/logging` 
		cfget_ret=$?
		if [ "$cfget_ret" = "0" -a "$logging" = "1" ]
		then
			echo -n "Shutting down clusterpro webmanager server: " > /dev/null 2>&1
		else
			echo -n "Shutting down clusterpro webmanager server: "
		fi
	fi

	#
	cd /opt/nec/clusterpro/bin

	# PIDFile for systemd
	[ -f ../work/service/clusterpro_webmgr.pid ] && { rm -f ../work/service/clusterpro_webmgr.pid >/dev/null 2>&1; }
	clp_logwrite "$1" "rm pid file." init_webmgr

	#
	clp_logwrite "$1" "clppidof start." init_webmgr
	pids=`/opt/nec/clusterpro/bin/clppidof /opt/nec/clusterpro/bin/clpmonp --webmgr 2>&1`
	clp_logwrite "$1" "clppidof end.(pids=${pids})" init_webmgr
	
	if [ "$pids" != "" ]
	then
		# shutting down clusterpro webmanager server
		clp_logwrite "$1" "webmgr stop process start." init_webmgr
		if [ "$targetos" = "SunOS" ]
		then
			kill -17 "$pids" 2>/dev/null
			kill_ret=$?
		else
			kill -12 "$pids" 2>/dev/null
			kill_ret=$?
		fi
		clp_logwrite "$1" "webmgr stop process end.("$kill_ret")" init_webmgr

		i=0
		while [ "$i" -lt "25" ]
		do
			clp_logwrite "$1" "clppidof2 start." init_webmgr
			pids=`/opt/nec/clusterpro/bin/clppidof /opt/nec/clusterpro/bin/clpmonp --webmgr 2>&1`
			clp_logwrite "$1" "clppidof2 end.(pids=${pids})" init_webmgr
			
			if [ "$pids" = "" ]
			then
				clp_logwrite "$1" "webmgr stop success." init_webmgr
				if [ "$cfget_ret" = "0" -a "$logging" = "1" ]
				then
					clp_success $targetos > /dev/null 2>&1
				else
					clp_success $targetos
				fi

				clp_logwrite "$1" "clusterpro_webmgr stop process end." init_webmgr
				exit 0
			fi

			sleep 1
			i=`expr "$i" + 1`
		done

		clp_logwrite "$1" "webmgr stop timeout." init_webmgr
	else
		clp_logwrite "$1" "webmgr not exist." init_webmgr
	fi

	clp_logwrite "$1" "clusterpro_webmgr stop process end." init_webmgr
	if [ "$cfget_ret" = "0" -a "$logging" = "1" ]
	then
		clp_success $targetos > /dev/null 2>&1
	else
		clp_success $targetos
	fi
	exit 0
	;;
  restart)
	$0 stop
	if [ "$?" != "0" ]
	then
		exit 1
	fi

	sleep 1
	$0 start
	;;
  status)
	targetos=`/bin/uname`
	if [ "$targetos" = "SunOS" ]
	then
		clp_status_s clpwebmc
	else
		clp_status clpwebmc
	fi
	;;
  *)
	echo "Usage: $0 {start|stop|restart|status}"
	exit 1
	;;
esac

exit 0
